%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Evaluation of Abaqus simulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
clear all;
clc;
warning('off','MATLAB:plot:IgnoreImaginaryXYPart')
set(0,'DefaultAxesFontSize',12)

disp('---- START: Evaluation of squirt simulation ----');
tic

SWITCH_WRITEFILE = 1;
SWITCH_SAVEFIGURE = 0;


% Time-dependent results
attribut = 'fs_8';
source_folder =['./' attribut '/'];
fid = fopen([source_folder 'ellipsesquirt-seq_results_fs.txt'],'r');
input_storage_size =  [30 Inf];
formatSpec = '%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f';
line_info = fgets(fid); E_s = str2num(line_info( strfind(line_info,':')+1 : end));
line_info = fgets(fid); K_f = str2num(line_info( strfind(line_info,':')+1 : end));
for ignore_iter=1:15 % Ignore first lines
fgets(fid);
end
line_info = fgets(fid); no_seq = str2num(line_info( strfind(line_info,':')+1 : end));
line_info = fgets(fid); n_to_def = str2num(line_info( strfind(line_info,':')+1 : end));
line_info = fgets(fid); n_to_dyn = str2num(line_info( strfind(line_info,':')+1 : end));
n_to_seq = 2*n_to_def + n_to_dyn;
line_info = fgets(fid); t_deform = str2num(line_info( strfind(line_info,':')+1 : end));
line_info = fgets(fid); t_relax_def = str2num(line_info( strfind(line_info,':')+1 : end));
line_info = fgets(fid); t_load = str2num(line_info( strfind(line_info,':')+1 : end));
line_info = fgets(fid); t_relax = str2num(line_info( strfind(line_info,':')+1 : end));
for ignore_iter=1:2 % Ignore further lines
fgets(fid);
end
input_storage = fscanf(fid,formatSpec,input_storage_size);
fclose(fid);
time = input_storage(1,:);
stress_xx = -input_storage(2,:);
strain_xx = -input_storage(3,:);    
stress_yy = input_storage(4,:);
strain_yy = input_storage(5,:);
f_c_preguess = input_storage(6,:);
rho_mixture = input_storage(7,:);
p_1 = input_storage(8,:);
p_2 = input_storage(9,:);
p_3 = input_storage(10,:);
p_4 = input_storage(11,:);
vol_1 = input_storage(12,:);
vol_2 = input_storage(13,:);
vol_3 = input_storage(14,:);
vol_4 = input_storage(15,:);
rf_xx = input_storage(16,:);
rf_xy = input_storage(17,:);
rf_yx = input_storage(18,:);
rf_yy = input_storage(19,:);
E_strain = input_storage(20,:);
E_strainartificial = input_storage(21,:);
E_kinetic = input_storage(22,:);
E_total = input_storage(23,:);
E_viscousdissip = input_storage(24,:);
E_externalwork = input_storage(25,:);
E_internal = input_storage(26,:);
disp_xx = input_storage(27,:);
disp_xy = input_storage(28,:);
disp_yx = input_storage(29,:);
disp_yy = input_storage(30,:);

index_space = linspace(1,length(time),length(time));
f_c = f_c_preguess(1);
prestrain_select = 9;

for i_seq = 1:no_seq+1
    disp(['Evluating sequence ' num2str(i_seq)]);
    % extract data
    interv_limit_lo = t_deform+t_relax_def + (t_deform+t_relax_def+t_load+t_relax)*(i_seq-1);
    interv_limit_hi = (t_deform+t_relax_def+t_load+t_relax)*i_seq;
    index_seq =       [n_to_seq*(i_seq-1) + 2*n_to_def + 1     : n_to_seq*(i_seq)];
    index_seq_focus = [n_to_seq*(i_seq-1) + 2*n_to_def + 1 - 1 : n_to_seq*(i_seq)];
    %disp(['seq index limits:   ' num2str(index_seq(1)) '   ' num2str(index_seq(end))]);
    if i_seq > 1 && index_seq(end) > length(time) 
        break 
        Qinv_SLS_max(i_seq) = NaN;
        f_SLS_max(i_seq) = NaN;
        Qinv_FFT_max(i_seq) = NaN;
        f_FFT_max(i_seq) = NaN;
        delta_p_check(i_seq) = NaN;
        stress_yy_slope_check(i_seq) = NaN;
        Qinv_max_check(i_seq) = NaN;
        f_max_check(i_seq) = NaN;
    end

    time_seq(i_seq,:) = time(index_seq);
    p_1_seq(i_seq,:) = p_1(index_seq);
    p_3_seq(i_seq,:) = p_3(index_seq);
    stress_yy_seq(i_seq,:) = stress_yy(index_seq);
    strain_yy_seq(i_seq,:) = strain_yy(index_seq);
    strain_currentdeform(i_seq) = strain_yy(index_seq(1));
    
    
    if prestrain_select == i_seq
        time_focus = time(index_seq_focus);
        stress_focus = stress_yy(index_seq_focus);
        strain_focus = strain_yy(index_seq_focus);
    end
    
    figure()
    subplot(2,2,1)
    plot(time_seq(i_seq,:),stress_yy_seq(i_seq,:),'LineWidth',2);
    xlabel('time');
    ylabel('s_{yy}');

    % Check if equilibrium is reached at start and end
    checkpoint_start_1 = index_seq(1)-1;
    checkpoint_start_2 = index_seq(1);
    checkpoint_end_1 = index_seq(end)-1;
    checkpoint_end_2 = index_seq(end);
    delta_p_start_1 = abs(p_1(checkpoint_start_1) - p_3(checkpoint_start_1));
    delta_p_start_2 = abs(p_1(checkpoint_start_2) - p_3(checkpoint_start_2));
    delta_p_end_1 = abs(p_1(checkpoint_end_1) - p_3(checkpoint_end_1));
    delta_p_end_2 = abs(p_1(checkpoint_end_2) - p_3(checkpoint_end_2));
    delta_p_max = max(abs(p_1(index_seq)-p_3(index_seq)));
    delta_p_check(i_seq) = (delta_p_start_1 + delta_p_start_2 + delta_p_end_1 + delta_p_end_2)/delta_p_max;
    stress_yy_slope_check(i_seq) = (abs(stress_yy(checkpoint_start_2)-stress_yy(checkpoint_start_1)) + ...
        abs(stress_yy(checkpoint_end_2)-stress_yy(checkpoint_end_1)) ) / ...
        abs(max(stress_yy(index_seq))-min(stress_yy(index_seq)));
    
    % Approximation by linear pressure relaxation
    strain_ev_withramp = abs(strain_yy_seq(i_seq,:)-strain_yy_seq(i_seq,1));
    [strain_ampl,index_relaxstart] = max(strain_ev_withramp);
    cutoff_timesteps = 0;
    stress_ev = abs(stress_yy_seq(i_seq,index_relaxstart:end-cutoff_timesteps)-stress_yy_seq(i_seq,1));
    strain_ev = abs(strain_yy_seq(i_seq,index_relaxstart:end-cutoff_timesteps)-strain_yy_seq(i_seq,1));
    time_ev = time_seq(i_seq,index_relaxstart:end-cutoff_timesteps) - time_seq(i_seq,index_relaxstart);
    [maxstress,index_maxstress] = max(stress_ev);
    sigma_0 = maxstress;
    index_evaluate_1 = floor(length(stress_ev)*7/40); % adapt to best quality
    sigma_end = stress_ev(end);
    delta_sigma = sigma_0-sigma_end;
    lambda = (-1)/time_ev(index_evaluate_1)*log((stress_ev(index_evaluate_1)-sigma_end)/(sigma_0-sigma_end));
    f_expfit = lambda/(2*pi);
    if f_expfit == 'NaN'
       f_expfit = 0; 
    end
    stress_SLS = sigma_end+delta_sigma*exp(-lambda*time_ev);
    Modulus_stat = sigma_end/strain_ampl;
    Modulus_dynp = delta_sigma/strain_ampl;
    f_Qinv_peak = f_expfit*sqrt(Modulus_stat/(Modulus_stat+Modulus_dynp));
    f_SLS = 10.^[-2:0.025:2];
    Qinv_SLS(i_seq,:) = (f_SLS./f_expfit).*(Modulus_dynp/Modulus_stat)./(1+ ((f_SLS/f_expfit).^2).*(1+Modulus_dynp/Modulus_stat));
    [Qinv_SLS_max(i_seq),Qinv_SLS_max_index] = max(Qinv_SLS(i_seq,:));
    f_SLS_max(i_seq) = f_SLS(Qinv_SLS_max_index);
    
    if prestrain_select == i_seq
        stress_SLS_focus = -stress_SLS + stress_focus(1);
        time_SLS_focus = time_ev + time_seq(i_seq,index_relaxstart);
        f_SLS_focus = f_SLS;
        Qinv_SLS_focus = Qinv_SLS(i_seq,:);
    end
    
        
    
    subplot(2,2,2)
    plot(time_ev,stress_ev,'LineWidth',2);
    hold on;
    plot(time_ev,stress_SLS,'r--','LineWidth',2);
    xlabel('time');
    ylabel('s_{yy}');
    
    
    % FFT
    [Qinv_FFT_temp, f_FFT_temp] = Q_inv_FFT(strain_ev,stress_ev,time_ev,50);
    if i_seq == 1
        Qinv_FFT(i_seq,:) = Qinv_FFT_temp;
        f_FFT(i_seq,:) = f_FFT_temp;
    else % FFT does not always return the same length, so cut or zero pad to length of first sequence
        if length(Qinv_FFT_temp) >= length(Qinv_FFT(1,:))
            Qinv_FFT(i_seq,:) = Qinv_FFT_temp(1:length(Qinv_FFT(1,:)));
            f_FFT(i_seq,:) = f_FFT_temp(1:length(Qinv_FFT(1,:)));
        else
            Qinv_FFT(i_seq,:) = [ Qinv_FFT_temp 0*linspace(1,1,length(Qinv_FFT(1,:)) - length(Qinv_FFT_temp)) ];
            f_FFT(i_seq,:) = [ f_FFT_temp 0*linspace(1,1,length(Qinv_FFT(1,:)) - length(Qinv_FFT_temp)) ];
        end
    end
    [Qinv_FFT_max(i_seq),Qinv_FFT_max_index] = max(Qinv_FFT(i_seq,1:length(Qinv_FFT)-3));
    f_FFT_max(i_seq) = f_FFT(i_seq,Qinv_FFT_max_index);
    
    if prestrain_select == i_seq
        f_FFT_focus = f_FFT(i_seq,:);
        Qinv_FFT_focus = Qinv_FFT(i_seq,:);
    end
    
    subplot(2,2,3)
    semilogx(f_SLS,Qinv_SLS(i_seq,:),'r--','LineWidth',2);
    hold on;
    semilogx(f_FFT(i_seq,1:min(length(f_FFT(i_seq,:)),12e2)),Qinv_FFT(i_seq,1:min(length(f_FFT(i_seq,:)),12e2)),'ko','MarkerSize',3);
    plot(f_SLS_max(i_seq),Qinv_SLS_max(i_seq),'rs');
    plot(f_FFT_max(i_seq),Qinv_FFT_max(i_seq),'ks');
    xlabel('f');
    ylabel('Q^{-1}');
    legend('SLS approximation','FFT');
    ylim([0.0,0.45]);
    
    % Check difference between FFT and SLS approximation
    Qinv_max_check(i_seq) = (Qinv_FFT_max(i_seq)-Qinv_SLS_max(i_seq))/Qinv_SLS_max(i_seq);
    f_max_check(i_seq) = (f_FFT_max(i_seq)-f_SLS_max(i_seq))/f_SLS_max(i_seq);
    

    
end

figure()
subplot(2,2,1)
plot(-strain_currentdeform,Qinv_SLS_max);
hold on;
plot(-strain_currentdeform,Qinv_FFT_max,'ks');
xlabel('compression strain');
ylabel('Q^{-1}_{max}');
legend('SLS-fit','FFT');
ylim([0,0.6]);
subplot(2,2,2)
plot(-strain_currentdeform,f_SLS_max/f_c);
hold on;
plot(-strain_currentdeform,f_FFT_max/f_c,'ks');
xlabel('compression strain');
ylabel('f_{max}/f_c');
subplot(2,2,3)
plot(-strain_currentdeform,Qinv_max_check);
hold on;
plot(-strain_currentdeform,f_max_check);
xlabel('compression strain');
ylabel('rel. error between SLS and FFT');
legend('Q^{-}_{max}','f_{max}');
subplot(2,2,4)
plot(-strain_currentdeform,delta_p_check);
hold on;
plot(-strain_currentdeform,stress_yy_slope_check);
xlabel('compression strain');
ylabel('equilibrium check');
legend('\Delta p','d{\sigma}_{yy}/dt');

if prestrain_select <= length(Qinv_SLS_max)
figure()
%
subplot(2,3,1);
plot(time*f_c,strain_yy,'k','LineWidth',2);
hold on;

plot(time_focus*f_c,strain_focus,'g','LineWidth',3);
hold off;
xlabel('t \cdot f_c');
ylabel('\epsilon');
new_axis = axes('position',[ 0.25 0.81 0.08 0.08]);
plot(time_focus*f_c,strain_focus,'k','LineWidth',2);
xlabel('t \cdot f_c');
ylabel('\epsilon');
ylim([strain_focus(end)+0.1*(strain_focus(end)-strain_focus(1)), strain_focus(1)-0.1*(strain_focus(end)-strain_focus(1))])
xlim([time_focus(1)*f_c, time_focus(end)*f_c]);
set(new_axis,'color','none');
%
subplot(2,3,2);
plot(time*f_c,stress_yy,'k','LineWidth',2);
hold on;
plot(time_focus*f_c,stress_focus,'g','LineWidth',3);
hold off;
xlabel('t \cdot f_c');
ylabel('\sigma');
new_axis2 = axes('position',[ 0.525 0.81 0.08 0.08]);
plot(time_focus*f_c,stress_focus,'k','LineWidth',2);
hold on;
plot(time_SLS_focus*f_c,stress_SLS_focus,'r--','LineWidth',3)
xlim([time_focus(1)*f_c, time_focus(end)*f_c]);
xlabel('t \cdot f_c');
ylabel('\sigma');
set(new_axis2,'color','none');
%
subplot(2,3,3);
hold on;
plot(f_FFT_focus./f_c,Qinv_FFT_focus,'k','LineWidth',2);
plot(f_SLS_focus./f_c,Qinv_SLS_focus,'r--','LineWidth',3)
xlabel('f/f_c');
ylabel('Q^{-1}');
legend('FFT','SLS');
set(gca,'XScale','log');


figure();
%
delta_sigma_dyn = abs(min(stress_focus)-stress_focus(end)); % info for comparison with other materials
delta_sigma_stat = abs(stress_focus(1)-stress_focus(end));
disp(['\delta \sigma_{stat} = ' num2str(delta_sigma_stat) '   |    \delta \sigma_{dyn} = ' num2str(delta_sigma_dyn) '   |   \delta \epsilon = ' num2str(strain_focus(1)-strain_focus(end)) '   |   f_c = ' num2str(f_SLS_max(prestrain_select))]);
%
subplot(2,1,1);
plot(time_focus*f_c,strain_focus,'k','LineWidth',2);
xlabel('t \cdot f_c');
ylabel('\epsilon');
ylim([strain_focus(end)+0.1*(strain_focus(end)-strain_focus(1)), strain_focus(1)-0.1*(strain_focus(end)-strain_focus(1))])
%
subplot(2,1,2);
plot(time_focus*f_c,stress_focus,'k','LineWidth',2);
hold on;
plot(time_SLS_focus*f_c,stress_SLS_focus,'r--','LineWidth',3);
xlabel('t \cdot f_c');
ylabel('\sigma');




figure()
%
[contact_strain, i_contact] = min(abs(strain_yy+0.11));
annotation('textbox',[.085 .64 .3 .3],'String','a)','FontSize',14,'FitBoxToText','on','LineStyle','none');
%
subplot(2,3,1);
plot(time(1:i_contact)*f_c,strain_yy(1:i_contact),'k','LineWidth',2);
hold on;
%plot(mean(time_focus)*f_c,mean(strain_focus),'s','MarkerSize',8,'LineWidth',1.5,'color','k');
annotation('textbox',[.134 .427 .3 .3],'String',' precompression step','FontSize',12,'FitBoxToText','on','LineStyle','none');
annotation('rectangle',[0.228 0.732 0.012 0.04],'FaceColor','blue','FaceAlpha',.2);
annotation('arrow',[0.228 0.205],[0.75 0.73],'HeadWidth',6,'HeadLength',6)
annotation('textbox',[.2430 .47 .3 .3],'String',' stress relaxation step','FontSize',12,'FitBoxToText','on','LineStyle','none');
annotation('rectangle',[0.24 0.732 0.005 0.012],'FaceColor','blue','FaceAlpha',.2);
annotation('arrow',[0.243 0.253],[0.745 0.81],'HeadWidth',6,'HeadLength',6)
hold off;
xlabel('t \cdot f_c [-]');
ylabel('$\varepsilon$ [-]','Interpreter','Latex');
ylim([strain_yy(i_contact),0]);
box on;
new_axis = axes('position',[ 0.255 0.83 0.08 0.08]);
plot(time_focus*f_c,strain_focus,'k','LineWidth',2);
ylabel('$\varepsilon$ [-]','Interpreter','Latex');
ylim([strain_focus(end)+0.1*(strain_focus(end)-strain_focus(1)), strain_focus(1)-0.1*(strain_focus(end)-strain_focus(1))])
xlim([time_focus(1)*f_c, time_focus(end)*f_c]);
%set(gca,'XTick',[floor(time_focus(1)*f_c):1:ceil(time_focus(end)*f_c)]+0.5);
%set(gca,'XTickLabel',[floor(time_focus(1)*f_c):1:ceil(time_focus(end)*f_c)]+0.5);
set(new_axis,'color','none');
%
subplot(2,3,4);
title('finite size: E_s/K_f = 100'); 
plot(time(1:i_contact)*f_c,stress_yy(1:i_contact)/E_s,'k','LineWidth',2);
hold on;
%plot(mean(time_focus)*f_c,mean(stress_focus)/E_s,'s','MarkerSize',8,'LineWidth',1.5,'color','k');
hold off;
xlabel('t \cdot f_c [-]');
ylabel('\sigma / E_s [-]');
ylim([stress_yy(i_contact)/E_s,0]);
box on;
new_axis2 = axes('position',[ 0.255 0.34 0.08 0.08]);
plot(time_SLS_focus*f_c,stress_SLS_focus/E_s,'r--','LineWidth',3)
hold on;
plot(time_focus*f_c,stress_focus/E_s,'k','LineWidth',2);
plot(time_SLS_focus*f_c,stress_SLS_focus/E_s,'r--','LineWidth',3)
legend('SLS-fit')
xlim([time_focus(1)*f_c, time_focus(end)*f_c]);
ylim([(min(stress_focus)-0.1*(max(stress_focus)-min(stress_focus)))/E_s, (max(stress_focus)+0.1*(max(stress_focus)-min(stress_focus)))/E_s]);
%xlabel('t \cdot f_c [-]');
ylabel('\sigma / E_s [-]');
%set(gca,'XTick',[floor(time_focus(1)*f_c):1:ceil(time_focus(end)*f_c)]+0.5);
%set(gca,'XTickLabel',[floor(time_focus(1)*f_c):1:ceil(time_focus(end)*f_c)]+0.5);
set(new_axis2,'color','none');
%
subplot(2,3,5);
hold on;
plot(f_FFT_focus./f_c,Qinv_FFT_focus,'b','LineWidth',2);
plot(f_SLS_focus./f_c,Qinv_SLS_focus,'r--','LineWidth',3)
xlabel('f/f_c [-]');
ylabel('Q^{-1} [-]');
legend('FFT','SLS-fit');
set(gca,'XScale','log');
xlim([10^-3,10^1]);
box on;
annotation('textbox',[.085 .17 .3 .3],'String','b)','FontSize',14,'FitBoxToText','on','LineStyle','none');
annotation('textbox',[.36  .17 .3 .3],'String','c)','FontSize',14,'FitBoxToText','on','LineStyle','none');

end


%% Save figure
set(gcf, 'Units', 'pixels', 'Position', [0, 0, 2000, 1000], 'PaperUnits', 'centimeters', 'PaperSize', [10.0, 10.0])
if SWITCH_SAVEFIGURE == 1
    saveas(gcf,'methodsteps_and_eval_fs','svg');
end




%% Write results into file
if SWITCH_WRITEFILE == 1
fileID = fopen(['Qfresults_' attribut '.txt'],'w');
fprintf(fid,['    compressive strain [-]' ...
             '           Qinv_SLS_max [-]' ...
             '             f_SLS_max [Hz]' ...
             '           Qinv_FFT_max [-]' ...
             '             f_FFT_max [Hz]' ...
             '          delta_p_check [-]' ...
             '  stress_yy_slope_check [-]' ...
             '         Qinv_max_check [-]' ...
             '            f_max_check [-]' ...
             '         f_c_predicted [Hz]' ...
             '        stiffness ratio [-]' ...
             '\n']);
for i_seq = 1:length(strain_currentdeform)
fprintf(fid,'%26.8f %26.8f %26.8f %26.8f %26.8f %26.8f %26.8f %26.8f %26.8f %26.8f %26.8f \n',-strain_currentdeform(i_seq),Qinv_SLS_max(i_seq),f_SLS_max(i_seq),Qinv_FFT_max(i_seq),f_FFT_max(i_seq),delta_p_check(i_seq),stress_yy_slope_check(i_seq),Qinv_max_check(i_seq),f_max_check(i_seq),f_c,E_s/K_f);
end
fclose(fid);
disp(['Result file written: buckling_localflow_finitesystem_EKratio_log10_' num2str(log10(E_s/K_f)) '.txt']);
end

disp('---- Finish: Evaluation of squirt simulation ----');
toc
