# #######################################################################
# ------------------------------- Preface ------------------------------
# #######################################################################
execfile('ellipsesquirt_predefinitions.py')
JobNameEvaluate = JobNameSequential
print("START: EVALUATION OF POSTBUCKLING RESULTS - SQUIRT FLOW / file is: " + JobNameEvaluate + "   (this may take a while)")

RHO_MIXTURE = []
Stress_XX = []
Stress_YY = []
StrainMacro_XX = [] # engineering strain
StrainMacro_YY = [] # engineering strain
disp_xx = []
disp_xy = []
disp_yx = []
disp_yy = []
TIME = []
P_1 = []
P_2 = []
P_3 = []
P_4 = []
VOL_1 = []
VOL_2 = []
VOL_3 = []
VOL_4 = []
RF_XX = []
RF_XY = []
RF_YX = []
RF_YY = []
ENERGY_STRAIN = []
ENERGY_STRAINARTIFICIAL = []
ENERGY_KINETIC = []
ENERGY_TOTAL = []
ENERGY_VISCOUSDISSIP = []
ENERGY_EXTERNALWORK = []
ENERGY_INTERNAL = []


# #######################################################################
# ---------------------------- Load PB Results --------------------------
# #######################################################################
print("   ... loading results ...")
o1 = session.openOdb(name = JobNameEvaluate +'.odb') # ATTENTION: Loading the odb-file makes letters upper case for many names, definitions, etc. due to regeneration of some things
All_Nodes = o1.rootAssembly.instances['INST-FULL'].nodeSets['ALL-NODES']
region_top = o1.rootAssembly.instances['INST-FULL'].nodeSets['Y+-SURF']
region_bot = o1.rootAssembly.instances['INST-FULL'].nodeSets['Y--SURF']
region_left = o1.rootAssembly.instances['INST-FULL'].nodeSets['X--SURF']
region_right = o1.rootAssembly.instances['INST-FULL'].nodeSets['X+-SURF']

Step_eval_name = 'Step-DS'

Strainenergy_state = o1.steps[Step_eval_name].historyRegions['Assembly ASSEMBLY'].historyOutputs['ALLSE'].data
Strainartificialenergy_state = o1.steps[Step_eval_name].historyRegions['Assembly ASSEMBLY'].historyOutputs['ALLAE'].data
Kineticenergy_state = o1.steps[Step_eval_name].historyRegions['Assembly ASSEMBLY'].historyOutputs['ALLKE'].data
Totalenergy_state = o1.steps[Step_eval_name].historyRegions['Assembly ASSEMBLY'].historyOutputs['ETOTAL'].data
Viscousdissipation_state = o1.steps[Step_eval_name].historyRegions['Assembly ASSEMBLY'].historyOutputs['ALLVD'].data
Externalworkenergy_state = o1.steps[Step_eval_name].historyRegions['Assembly ASSEMBLY'].historyOutputs['ALLWK'].data
Internalenergy_state = o1.steps[Step_eval_name].historyRegions['Assembly ASSEMBLY'].historyOutputs['ALLIE'].data

P_1_state = o1.steps[Step_eval_name].historyRegions['Node REFPRESSURE_R3C3.1'].historyOutputs['PCAV'].data
P_2_state = o1.steps[Step_eval_name].historyRegions['Node REFPRESSURE_R3C4.1'].historyOutputs['PCAV'].data
P_3_state = o1.steps[Step_eval_name].historyRegions['Node REFPRESSURE_R4C3.1'].historyOutputs['PCAV'].data
P_4_state = o1.steps[Step_eval_name].historyRegions['Node REFPRESSURE_R4C4.1'].historyOutputs['PCAV'].data
VOL_1_state = o1.steps[Step_eval_name].historyRegions['Node REFPRESSURE_R3C3.1'].historyOutputs['CVOL'].data
VOL_2_state = o1.steps[Step_eval_name].historyRegions['Node REFPRESSURE_R3C4.1'].historyOutputs['CVOL'].data
VOL_3_state = o1.steps[Step_eval_name].historyRegions['Node REFPRESSURE_R4C3.1'].historyOutputs['CVOL'].data
VOL_4_state = o1.steps[Step_eval_name].historyRegions['Node REFPRESSURE_R4C4.1'].historyOutputs['CVOL'].data



# use the following command in the Abaqus-Python-shell (the bottom one in the GUI) to see a list of allowed history regions: print o1.steps['Step-DS'].historyRegions.keys() 



# #######################################################################
# ----------------------- Compute Data of Interest ----------------------
# #######################################################################
print("   ... computing data ...")

# sum up total reaction force:
nodes_top =  o1.rootAssembly.instances['INST-FULL'].nodeSets['Y+-SURF'].nodes
no_nodes_top = len(nodes_top)
RF2_all = (    numpy.array( o1.steps[Step_eval_name].historyRegions['Node INST-FULL.' + str(nodes_top[0].label)].historyOutputs['RF2'].data )   )[:,1]
for i_node in range(1,no_nodes_top):
    print("   processing node " + str(i_node+1) + " of " + str(no_nodes_top))
    RF2_currentnode = (    numpy.array( o1.steps[Step_eval_name].historyRegions['Node INST-FULL.' + str(nodes_top[i_node].label)].historyOutputs['RF2'].data )   )[:,1]
    RF2_all = RF2_all + RF2_currentnode

# time loop
for i in range(len(Strainenergy_state)):

    ReactForce_XX = NaN
    ReactForce_XY = NaN
    ReactForce_YX = NaN
    ReactForce_YY = RF2_all[i]
    Disp_VP_X  = NaN
    Disp_VP_XY = NaN
    Disp_VP_YX = NaN
    Disp_VP_Y  = o1.steps[Step_eval_name].historyRegions['Node INST-FULL.' + str(nodes_top[i_node].label)].historyOutputs['U2'].data[i][1]

    disp_xx.append(Disp_VP_X)
    disp_xy.append(Disp_VP_XY)
    StrainMacro_XX_step = Disp_VP_X/(no_holes_hor*d_cell)
    StrainMacro_XX.append(-abs(StrainMacro_XX_step))
    disp_yx.append(Disp_VP_YX)
    disp_yy.append(Disp_VP_Y)
    StrainMacro_YY_step = Disp_VP_Y/(no_holes_ver*h_cell)
    StrainMacro_YY.append(-abs(StrainMacro_YY_step))

    RF_XX.append(ReactForce_XX)
    RF_XY.append(ReactForce_XY)
    RF_YX.append(ReactForce_YX)
    RF_YY.append(ReactForce_YY)
    surface_total_X = (h_cell*no_holes_ver*depth)
    Stress_XX.append(ReactForce_XX/surface_total_X)
    surface_total = (d_cell*no_holes_hor)*depth
    Stress_YY.append(ReactForce_YY/surface_total)



    P_1.append(P_1_state[i][1])
    P_2.append(P_2_state[i][1])
    P_3.append(P_3_state[i][1])
    P_4.append(P_4_state[i][1])

    VOL_1.append(VOL_1_state[i][1])
    VOL_2.append(VOL_2_state[i][1])
    VOL_3.append(VOL_3_state[i][1])
    VOL_4.append(VOL_4_state[i][1])

    #Geometry evaluation turned out to be too costly for finite size sample in a reasonable time (plus fewer time steps are available with full field information)
    #evol = o1.steps[Step_eval_name].frames[i].fieldOutputs['EVOL']
    #number_elements = len(evol.values)
    #volume_structure = 0.0
    #for iter_element in range(0,number_elements): volume_structure = volume_structure + evol.values[iter_element].data
    #volume_total = 2.0*h_cell * 2.0*d_cell * depth * (1+min(StrainMacro_XX_step,StrainMacro_YY_step))
    #porosity = 1 - volume_structure/volume_total
    #RHO_MIXTURE.append((rho_1+rho_2+rho_3+rho_4)/(4.0)*porosity + Rho/rho_mod*(1-porosity))
    RHO_MIXTURE.append(NaN)

    TIME.append(Strainenergy_state[i][0])

    ENERGY_STRAIN.append(Strainenergy_state[i][1])
    ENERGY_STRAINARTIFICIAL.append(Strainartificialenergy_state[i][1])
    ENERGY_KINETIC.append(Kineticenergy_state[i][1])
    ENERGY_TOTAL.append(Totalenergy_state[i][1])
    ENERGY_VISCOUSDISSIP.append(Viscousdissipation_state[i][1])
    ENERGY_EXTERNALWORK.append(Externalworkenergy_state[i][1])
    ENERGY_INTERNAL.append(Internalenergy_state[i][1])





# #######################################################################
# ---------------- Write PB Results for later Matlab use ----------------
# #######################################################################
print("   ... writing file ...")

Output_file = open(JobNameEvaluate + '_results_fs.txt', 'w')
Output_file.write('Stiffness E0: %10.8g \n' % (E0))
Output_file.write('Stiffness K_f_1: %10.8g \n' % (K_f_1))
Output_file.write('Stiffness K_f_2: %10.8g \n' % (K_f_2))
Output_file.write('Stiffness K_f_3: %10.8g \n' % (K_f_3))
Output_file.write('Stiffness K_f_4: %10.8g \n' % (K_f_4))
Output_file.write('Porosity: %10.8g \n' % porosity)
Output_file.write('A_ellipse: %10.8g \n' % (0.25*pi*d_ellipse*h_ellipse))
Output_file.write('cell height: %10.8g \n' % (h_cell))
Output_file.write('cell width: %10.8g \n' % (d_cell))
Output_file.write('ellipse height: %10.8g \n' % (h_ellipse))
Output_file.write('ellipse width: %10.8g \n' % (d_ellipse))
Output_file.write('- \n')
Output_file.write('P_average_end (1-3): %10.8g \n' % ((P_1[-1]+P_3[-1])/2.0))
Output_file.write('Delta_P/P_end (1-3): %10.8g \n' % ( absolute((P_3[-1]-P_1[-1])/((P_3[-1]+P_1[-1])/2.0)) ))
Output_file.write('P_average_end (2-4): %10.8g \n' % ((P_2[-1]+P_4[-1])/2.0))
Output_file.write('Delta_P/P_end (2-4): %10.8g \n' % ( absolute((P_2[-1]-P_4[-1])/((P_2[-1]+P_4[-1])/2.0)) ))
Output_file.write('- \n')
Output_file.write('Number of sequences: %i \n' % no_seq)
Output_file.write('Number of output intervals (deformation, deform and relax each): %i \n' % no_output_interv_def)
Output_file.write('Number of output intervals (dynamic, incl. both perturb and relax): %i \n' % no_output_interv_dyn)
Output_file.write('t_deform: %10.8g \n' % t_deform)
Output_file.write('t_relax_def: %10.8g \n' % t_relax_def)
Output_file.write('t_load: %10.8g \n' % t_load)
Output_file.write('t_relax: %10.8g \n' % t_relax)

Output_file.write('--- \n')
Output_file.write('time    Stress_XX    StrainMacro_XX    Stress_YY    StrainMacro_YY    f_c    rho_mixture    P_1    P_2    P_3    P_4    VOL_1    VOL_2    VOL_3    VOL_4    RF_XX    RF_XY    RF_YX    RF_YY    ENERGY_STRAIN    ENERGY_STRAINARTIFICIAL    ENERGY_KINETIC    ENERGY_TOTAL    ENERGY_VISCOUSDISSIP    ENERGY_EXTERNALWORK    ENERGY_INTERNAL    disp_xx    disp_xy    disp_yx    disp_yy \n')
for i in range(len(StrainMacro_YY)):
	Output_file.write('%10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g %10.8g \n' % 
        (TIME[i], 
	Stress_XX[i], 
	StrainMacro_XX[i], 
	Stress_YY[i], 
	StrainMacro_YY[i], 
	f_c, 
	RHO_MIXTURE[i], 
	P_1[i], 
	P_2[i], 
	P_3[i], 
	P_4[i], 
	VOL_1[i], 
	VOL_2[i], 
	VOL_3[i], 
	VOL_4[i], 
	RF_XX[i], 
	RF_XY[i], 
	RF_YX[i], 
	RF_YY[i],
	ENERGY_STRAIN[i],
	ENERGY_STRAINARTIFICIAL[i],
	ENERGY_KINETIC[i],
	ENERGY_TOTAL[i],
	ENERGY_VISCOUSDISSIP[i],
	ENERGY_EXTERNALWORK[i],
	ENERGY_INTERNAL[i],
	disp_xx[i],
	disp_xy[i],
	disp_yx[i],
	disp_yy[i]))
Output_file.close()


o1.close()

print("FINISH: EVALUATION OF POSTBUCKLING RESULTS - SQUIRT FLOW")

