########################################################################
# ------------------------------ Preface -------------------------------
########################################################################
execfile('ellipsesquirt_predefinitions.py')
execfile('ellipsesquirt_generatemodel_wet.py')
print("START")



########################################################################
# ------------------------- Open Basic Model ---------------------------
########################################################################
mdb = openMdb(pathName= JobNameModelwet)
	# and because Abaqus does not seem to store the manually defined but valuable variables in the CAE-file, here we go again manually:
Model_2D = mdb.models[ModelName]
NameRef1='RefPoint-0'
NameRef2='RefPoint-1'
Part_Full = Model_2D.parts['Part-Full']


########################################################################
# ----------- Load & Apply Imperfections ----------------
# to trigger a buckling mode, e.g., by random mesh imperfections
# optional, but not used in final simulations
########################################################################
# NewCoord=numpy.zeros((len(Part_Full.nodes), 3), float)
# TOL_nodes = 1e-6
# readfile = open(ModelName + '_randomizednodedispl.py', 'r')
# lines = [line.rstrip('\n') for line in readfile]
# readfile.close()
# for i in range(len(Part_Full.nodes)):
#     random_imp_x = float(lines[i*2])
#     random_imp_y = float(lines[i*2+1])
#     NewCoord[i][0]=Part_Full.nodes[i].coordinates[0] + IMP_RD*random_imp_x
#     NewCoord[i][1]=Part_Full.nodes[i].coordinates[1] + IMP_RD*random_imp_y
# Part_Full.editNode(nodes=Part_Full.nodes, coordinates=NewCoord)
# Model_2D.rootAssembly.regenerate()   





########################################################################
# ------------------------------ Material ------------------------------
########################################################################
G0 = E0/(2.0*(1+Nu0))
K0 = 2.0*(1.0 + Nu0)*G0/(3.0*(1.0 - 2.0*Nu0))
C10 = G0/2.0 
D1 = 2.0/K0

Neo_Hook_Mat = Model_2D.Material(name='Mat-NeoHook-Rubber')
Neo_Hook_Mat.Hyperelastic(table=((C10, D1), ), testData=OFF, type=NEO_HOOKE, volumetricResponse=VOLUMETRIC_DATA)
Neo_Hook_Mat.Density(table=((Rho, ), ))


########################################################################
# ------------------------------ Amplitude for dynamic perturbation ------------------------------
########################################################################
amp_norm = no_seq*(1.0)
amp_data =()
amp_data = amp_data + ((0.0, 0.001),) # starting point, initial perturbation required for numerical start
for i_ad in range(0,no_seq+1):
    amp_data = amp_data + (
    ( t_deform*(i_ad+1)+t_relax_def*(i_ad)  +t_load*(i_ad)  +t_relax*(i_ad)   , 1.0*i_ad/amp_norm ),
    ( t_deform*(i_ad+1)+t_relax_def*(i_ad+1)+t_load*(i_ad)  +t_relax*(i_ad)   , 1.0*i_ad/amp_norm ),
    ( t_deform*(i_ad+1)+t_relax_def*(i_ad+1)+t_load*(i_ad+1)+t_relax*(i_ad)   , (1.0*i_ad+perturb_amp)/amp_norm ),
    ( t_deform*(i_ad+1)+t_relax_def*(i_ad+1)+t_load*(i_ad+1)+t_relax*(i_ad+1) , (1.0*i_ad+perturb_amp)/amp_norm ),
    )
Model_2D.SmoothStepAmplitude(data=amp_data, name='Amp-seq', timeSpan=STEP)


# output time steps
t_out_data = ()
t_out_data = t_out_data + ((0.0, ), )
for i_seq in range(0,no_seq+1):
    for i_to in range(1,no_output_interv_def+1):
        t_out_add = amp_data[i_seq*4][0] + t_deform/no_output_interv_def*i_to
        t_out_data = t_out_data + ((t_out_add,), )
    for i_to in range(1,no_output_interv_def+1):
        t_out_add = amp_data[i_seq*4+1][0] + t_relax_def/no_output_interv_def*i_to
        t_out_data = t_out_data + ((t_out_add,), )
    for i_to in range(1,no_output_interv_dyn+1):
        t_out_add = amp_data[i_seq*4+2][0] + (t_load+t_relax)/no_output_interv_dyn*i_to
        t_out_data = t_out_data + ((t_out_add,), )

t_out_data_fields = ()
for i_toutfield in range(0,size(t_out_data)):
	if i_toutfield%10 == 0:
		t_out_data_fields = t_out_data_fields + (t_out_data[i_toutfield], )


Model_2D.TimePoint(name='TimePoints-out', points=t_out_data)
Model_2D.TimePoint(name='TimePoints-out-fields', points=t_out_data_fields)




	
########################################################################
# ------------------------------- STEPS --------------------------------
########################################################################


# Dynamic perturbation
StepDSname = 'Step-DS'
#    create
Model_2D.ImplicitDynamicsStep(description='Sequential Uniaxial Compression', name=StepDSname, previous='Initial', 
	nlgeom=ON, maxNumInc=incrementnumber_max, minInc=incrementstep_min, initialInc=incrementstep_initial, maxInc=incrementstep_max, timePeriod=amp_data[-1][0])
#    output requests
Model_2D.FieldOutputRequest(createStepName=StepDSname, name='F-Output'+StepDSname, 
		variables=('S','E','LE','U','RF','CF','EVOL'), timePoint='TimePoints-out-fields')
Model_2D.HistoryOutputRequest(createStepName=StepDSname, name='H-Output'+StepDSname,
		variables=('ALLEN',), timePoint='TimePoints-out')
Model_2D.HistoryOutputRequest(createStepName=StepDSname, name='H-Output'+StepDSname+'-surfytop', 
		rebar=EXCLUDE, region=mdb.models['ellipsesquirt'].rootAssembly.allInstances['Inst-Full'].sets['Y+-Surf'], sectionPoints=DEFAULT, 
		timePoint='TimePoints-out', variables=('S11', 'S22', 'S12', 'SP', 'PRESS', 'U1', 'U2', 'RF1', 'RF2', 'RM3', 'RT'))
for cavity_hor_iter in range (1,no_holes_hor+1,):
    for cavity_ver_iter in range (1,no_holes_ver+1,):
        refpressure_name = 'RefPressure_' + 'r' + str(cavity_ver_iter) + 'c' + str(cavity_hor_iter)
        cavity_houtput_name = 'H-Output-FluidCavity_' + 'r' + str(cavity_ver_iter) + 'c' + str(cavity_hor_iter)
        Model_2D.HistoryOutputRequest(createStepName=StepDSname, name=cavity_houtput_name, rebar=EXCLUDE, region=Model_2D.rootAssembly.sets[refpressure_name], sectionPoints=DEFAULT, variables=('PCAV', 'MFL1', 'MFL2', 'MFL3', 'MFLM', 'MFLT', 'CVOL'), timePoint='TimePoints-out')
del Model_2D.fieldOutputRequests['F-Output-1']
del Model_2D.historyOutputRequests['H-Output-1']

#    BC
Model_2D.DisplacementBC(amplitude=UNSET, createStepName=StepDSname, distributionType=UNIFORM, fieldName='', localCsys=None, name=
    'BC-bottom', region=Model_2D.rootAssembly.instances['Inst-Full'].sets['Y--Surf'], u1=UNSET, u2=SET, ur3=UNSET)
Model_2D.DisplacementBC(amplitude=UNSET, createStepName=StepDSname, distributionType=UNIFORM, fieldName='', localCsys=None, name=
    'BC-left', region=Model_2D.rootAssembly.instances['Inst-Full'].sets['X--Surf'], u1=SET, u2=UNSET, ur3=UNSET)
Model_2D.DisplacementBC(amplitude=UNSET, createStepName=StepDSname, distributionType=UNIFORM, fieldName='', localCsys=None, name=
    'BC-right', region=Model_2D.rootAssembly.instances['Inst-Full'].sets['X+-Surf'], u1=SET, u2=UNSET, ur3=UNSET)
Model_2D.DisplacementBC(amplitude='Amp-seq', createStepName=StepDSname, distributionType=UNIFORM, fieldName='', localCsys=None, name=
    'BC-top', region=Model_2D.rootAssembly.instances['Inst-Full'].sets['Y+-Surf'], u1=UNSET, u2=bc_disp_total, ur3=UNSET)




	
########################################################################
# --------------------------------- Job --------------------------------
########################################################################	
# Create
mdb.Job(atTime=None, contactPrint=OFF, description='', echoPrint=OFF, 
    explicitPrecision=SINGLE, getMemoryFromAnalysis=True, historyPrint=OFF, 
    memory=num_ram, memoryUnits=unit_ram, model=Model_2D, 
    modelPrint=OFF, multiprocessingMode=DEFAULT, name=JobNameSequential, 
    nodalOutputPrecision=FULL, numCpus=num_cpu, numDomains=num_cpu, queue=None, scratch=Scratch_Dir, 
    type=ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)	
print("POST BUCKLING CREATION COMPLETE - NOW SAVING AND SUBMITTING")
# Save as a Model, in case later calculation crashes
mdb.saveAs(pathName = JobNameSequential  +'.cae')
# Submit
mdb.jobs[JobNameSequential].submit(consistencyChecking=OFF)
mdb.jobs[JobNameSequential].waitForCompletion()


########################################################################
# ------------------------------ Evaluation ----------------------------
########################################################################
execfile('ellipsesquirt_evaluate_output_FS.py')

print("FINISH")
