########################################################################
# ------------------------------ Preface -------------------------------
########################################################################
execfile('ellipsesquirt_predefinitions.py')
print("START: GENERATE MODEL")



########################################################################
# -------------------- Name settings and adjustments -------------------
########################################################################
mdb.models.changeKey(fromName='Model-1', toName=ModelName) 	
Model_2D = mdb.models[ModelName]



########################################################################
# ------------------------------ Create Geometry -----------------------
########################################################################

Model_2D.ConstrainedSketch(name='__profile__', sheetSize=1.0)
Model_2D.sketches['__profile__'].rectangle(point1=(-d_cell/2.0, -h_cell/2.0), point2=(0.0, 0.0))
Model_2D.sketches['__profile__'].EllipseByCenterPerimeter(axisPoint1=(d_ellipse/2.0, 0.0), axisPoint2=(0.0, h_ellipse/2.0), center=(0.0, 0.0))
Model_2D.sketches['__profile__'].autoTrimCurve(curve1=Model_2D.sketches['__profile__'].geometry.findAt((0.0, h_ellipse/2.0), ), point1=(0.0, h_ellipse/2.0))
Model_2D.sketches['__profile__'].autoTrimCurve(curve1=Model_2D.sketches['__profile__'].geometry.findAt((d_ellipse/2.0, 0.0), ), point1=(d_ellipse/2.0, 0.0))
Model_2D.sketches['__profile__'].autoTrimCurve(curve1=Model_2D.sketches['__profile__'].geometry.findAt((0.0, -TOL), ), point1=(0.0, -TOL))
Model_2D.sketches['__profile__'].autoTrimCurve(curve1=Model_2D.sketches['__profile__'].geometry.findAt((-TOL, 0.0), ), point1=(-TOL, 0.0))
Model_2D.Part(dimensionality=TWO_D_PLANAR, name='Part-1', type=DEFORMABLE_BODY)
Model_2D.parts['Part-1'].BaseShell(sketch=Model_2D.sketches['__profile__'])
del Model_2D.sketches['__profile__']




########################################################################
# ------------------------------ MESH AND MIRRORED PART -----------------------
########################################################################
# Mesh first part and create two copies (one mirrored)
Model_2D.parts['Part-1'].seedPart(deviationFactor=M_Devi, minSizeFactor=Min_M_Size, size=M_Size)
mdb.models['ellipsesquirt'].parts['Part-1'].setMeshControls(elemShape=TRI, regions=mdb.models['ellipsesquirt'].parts['Part-1'].faces.findAt(((-d_cell/2.0+TOL, -h_cell/2.0+TOL, 0.0), )))
Model_2D.parts['Part-1'].setElementType(elemTypes=(ElemType(elemCode=CPE4R, elemLibrary=STANDARD), ), regions=(Model_2D.parts['Part-1'].faces.findAt(((-d_cell/2.0+TOL, -h_cell/2.0+TOL, 0.0), )), ))
Model_2D.parts['Part-1'].generateMesh()

Part_2D_Mesh = Model_2D.parts['Part-1'].PartFromMesh(name='Part-1-mesh-1')
Part_2D_Mesh_MirrorX = Model_2D.Part(compressFeatureList=ON, mirrorPlane=YZPLANE, name='Part-1-mesh-2', objectToCopy=Part_2D_Mesh)
Part_2D_Mesh_MirrorY1 = Model_2D.Part(compressFeatureList=ON, mirrorPlane=XZPLANE, name='Part-1-mesh-3', objectToCopy=Part_2D_Mesh)
Part_2D_Mesh_MirrorY2 = Model_2D.Part(compressFeatureList=ON, mirrorPlane=XZPLANE, name='Part-1-mesh-4', objectToCopy=Part_2D_Mesh_MirrorX)

	
########################################################################
# ------------------------------ INSTANCE AND ASSEMBLY -----------------------
########################################################################

Model_2D.rootAssembly.DatumCsysByDefault(CARTESIAN)
Model_2D.rootAssembly.Instance(dependent=ON, name='Part-1-mesh-1', part=Part_2D_Mesh)
Model_2D.rootAssembly.Instance(dependent=ON, name='Part-1-mesh-2', part=Part_2D_Mesh_MirrorX)
Model_2D.rootAssembly.Instance(dependent=ON, name='Part-1-mesh-3', part=Part_2D_Mesh_MirrorY1)
Model_2D.rootAssembly.Instance(dependent=ON, name='Part-1-mesh-4', part=Part_2D_Mesh_MirrorY2)



    



# ########################################################################
# # --------------------- Linear Pattern NxN --------------------
# ########################################################################

Model_2D.rootAssembly.LinearInstancePattern(direction1=(1.0, 0.0, 0.0), direction2=(0.0, 1.0, 0.0), 
    instanceList=('Part-1-mesh-3', 'Part-1-mesh-1', 'Part-1-mesh-2', 'Part-1-mesh-4'), number1=no_holes_hor, number2=no_holes_ver, spacing1=d_cell, spacing2=h_cell)
List_Inst_Names = []
for quarter_iter in range(1,5):
    instance_name = 'Part-1-mesh-' + str(quarter_iter)
    List_Inst_Names.append(instance_name)
    for pattern_hor_iter in range (1,no_holes_hor+1):
        for pattern_ver_iter in range (1,no_holes_ver+1):
            if pattern_hor_iter != 1 or pattern_ver_iter != 1:
                instance_name = 'Part-1-mesh-' + str(quarter_iter) + '-lin-' + str(pattern_hor_iter) + '-' + str(pattern_ver_iter)
                List_Inst_Names.append(instance_name)
All_Instances = []
for instance_iter in range(0,size(List_Inst_Names)):
    All_Instances.append(Model_2D.rootAssembly.instances[List_Inst_Names[instance_iter]])

Part_Full = Model_2D.rootAssembly.PartFromBooleanMerge(domain=MESH, instances=All_Instances, mergeNodes=BOUNDARY_ONLY, name='Part-Full', nodeMergingTolerance=TOL)
Inst_Full = Model_2D.rootAssembly.Instance(dependent=ON, name='Inst-Full', part=Part_Full)
Model_2D.rootAssembly.suppressFeatures(List_Inst_Names)    






# ########################################################################
# # ------------------------------ Material ------------------------------
# ########################################################################
G0 = E0/(2.0*(1+Nu0))
K0 = 2.0*(1.0 + Nu0)*G0/(3.0*(1.0 - 2.0*Nu0))
C10 = G0/2.0 
D1 = 2.0/K0

Neo_Hook_Mat = Model_2D.Material(name='Mat-NeoHook-Rubber')
Neo_Hook_Mat.Hyperelastic(table=((C10, D1), ), testData=OFF, type=NEO_HOOKE, volumetricResponse=VOLUMETRIC_DATA)
Neo_Hook_Mat.Density(table=((Rho, ), ))



# ########################################################################
# # ------------------------------ Section -------------------------------
# ########################################################################
Section_NeoHook = Model_2D.HomogeneousSolidSection(material='Mat-NeoHook-Rubber', name='Section-NeoHook', thickness=None)
Part_Full.SectionAssignment(offset=0.0, offsetField='', offsetType=MIDDLE_SURFACE, region=Region(elements=Part_Full.elements), 
    sectionName='Section-NeoHook', thicknessAssignment=FROM_SECTION)





########################################################################
# ------------------ Define Sets of Nodes for BC -----------------------
########################################################################
	
Part_Full.Set(name='All-Elems', elements=Part_Full.elements) 
Part_Full.Set(name='All-Nodes', nodes=Part_Full.nodes) 



Part_Full.Set(name='X--Surf', nodes=Part_Full.nodes.getByBoundingBox(x_bot - TOL, y_bot - TOL, z_bot - TOL, x_bot + TOL, y_top + TOL, z_top + TOL))  
Part_Full.Set(name='X+-Surf', nodes=Part_Full.nodes.getByBoundingBox(x_top - TOL, y_bot - TOL, z_bot - TOL, x_top + TOL, y_top + TOL, z_top + TOL)) 
Part_Full.Set(name='Y--Surf', nodes=Part_Full.nodes.getByBoundingBox(x_bot - TOL, y_bot - TOL, z_bot - TOL, x_top + TOL, y_bot + TOL, z_top + TOL))  
Part_Full.Set(name='Y+-Surf', nodes=Part_Full.nodes.getByBoundingBox(x_bot - TOL, y_top - TOL, z_bot - TOL, x_top + TOL, y_top + TOL, z_top + TOL)) 
Model_2D.rootAssembly.SetByBoolean(name='PerBound'
    , sets=(
    Model_2D.rootAssembly.instances['Inst-Full'].sets['X+-Surf'], 
    Model_2D.rootAssembly.instances['Inst-Full'].sets['X--Surf'], 
    Model_2D.rootAssembly.instances['Inst-Full'].sets['Y+-Surf'], 
    Model_2D.rootAssembly.instances['Inst-Full'].sets['Y--Surf']))

Fix_point = ((x_bot+x_top)/2.0,(y_bot+y_top)/2.0,(z_bot+z_top)/2.0)
Part_Full.Set(name='Set-Fixpoint', nodes=Part_Full.nodes.getByBoundingBox(Fix_point[0] - TOL, Fix_point[1] - TOL, Fix_point[2]  - TOL, Fix_point[0]  + TOL, Fix_point[1] + TOL, Fix_point[2] + TOL)) 


	# Constraints of periodic boundary conditions
# not used in finite size model


# Fluid cavities
#   reference pressure points
for pattern_hor_iter in range (1,no_holes_hor+1):
    for pattern_ver_iter in range (1,no_holes_ver+1):
        refpressurename = 'RefPressure' + '_r' + str(pattern_ver_iter) + 'c' + str(pattern_hor_iter)
        refpressure_xcoord = x_bot + d_cell*0.5 + (pattern_hor_iter-1)*d_cell
        refpressure_ycoord = y_bot + h_cell*0.5 + (pattern_ver_iter-1)*h_cell
        Model_2D.Part(dimensionality=TWO_D_PLANAR, name=refpressurename, type=DEFORMABLE_BODY)
        Model_2D.parts[refpressurename].ReferencePoint(point=(refpressure_xcoord, refpressure_ycoord, 0.0))
        Model_2D.rootAssembly.Instance(dependent=ON, name=refpressurename, part=Model_2D.parts[refpressurename])
        Model_2D.rootAssembly.Set(name=refpressurename, referencePoints=(Model_2D.rootAssembly.instances[refpressurename].referencePoints[1],))    

#   surfaces (attention: since finding by bounding box or other simple ways is not available using Phython in the current version of Abaqus, the element faces must be determined using the GUI for the original surfaces and the nearest surface neighbours in x and y direction and separately saved to the external file surface_construction.py from the .jnl)
execfile('surface_construction_generateallsurfaces.py')
execfile('surface_construction_generateallsurfaces.py') # 2x, because the Abaqus-Python combination only exports/recognizes a text file after the 2nd time
mdb.models['ellipsesquirt'].rootAssembly.regenerate()
execfile('surface_construction_all.py')
mdb.models['ellipsesquirt'].rootAssembly.regenerate()





########################################################################
# -------------------- cavity properties -------------------
########################################################################
# for periodic patterns of
# --------------------
# |(3)||(4)||(3)||(4)|
# --------------------
# |(1)||(2)||(1)||(2)|
# --------------------
# --------------------
# |(3)||(4)||(3)||(4)|
# --------------------
# |(1)||(2)||(1)||(2)|
# --------------------
Model_2D.setValues(absoluteZero=absolute0, universalGas=universalGasConstant)
for cavity_hor_iter in range (1,no_holes_hor+1,2):
    for cavity_ver_iter in range (1,no_holes_ver+1,2):
        alternating_switch = 0.0
        if alternating_pattern == 1:
            if (cavity_ver_iter-1)/2%2==0:
                alternating_switch = 0.0
            else:
                alternating_switch = 1.0
        append_name_1 = 'r' + str(cavity_ver_iter) + 'c' + str(cavity_hor_iter)
        append_name_2 = 'r' + str(cavity_ver_iter) + 'c' + str(cavity_hor_iter+1)
        append_name_3 = 'r' + str(cavity_ver_iter+1) + 'c' + str(cavity_hor_iter)
        append_name_4 = 'r' + str(cavity_ver_iter+1) + 'c' + str(cavity_hor_iter+1)
        cavity_name_1 = 'FluidCavity_' + append_name_1
        cavity_name_2 = 'FluidCavity_' + append_name_2
        cavity_name_3 = 'FluidCavity_' + append_name_3
        cavity_name_4 = 'FluidCavity_' + append_name_4
        Model_2D.FluidCavityProperty(definition=PNEUMATIC,molecularWeight=MW_air, name='IntProp-' + cavity_name_1)
        Model_2D.FluidCavityProperty(definition=PNEUMATIC,molecularWeight=MW_air, name='IntProp-' + cavity_name_2)
        Model_2D.FluidCavityProperty(definition=PNEUMATIC,molecularWeight=MW_air, name='IntProp-' + cavity_name_3)
        Model_2D.FluidCavityProperty(definition=PNEUMATIC,molecularWeight=MW_air, name='IntProp-' + cavity_name_4)
        Model_2D.interactionProperties['IntProp-' + cavity_name_1].setValues(bulkModulusTable=((K_f_1*(1-alternating_switch) + K_f_2*alternating_switch, ), ), definition=HYDRAULIC, fluidDensity=rho_1*(1-alternating_switch)+rho_2*alternating_switch, molecularWeight=None, useBulkModulus=True)
        Model_2D.interactionProperties['IntProp-' + cavity_name_2].setValues(bulkModulusTable=((K_f_2*(1-alternating_switch) + K_f_1*alternating_switch, ), ), definition=HYDRAULIC, fluidDensity=rho_2*(1-alternating_switch)+rho_1*alternating_switch, molecularWeight=None, useBulkModulus=True)
        Model_2D.interactionProperties['IntProp-' + cavity_name_3].setValues(bulkModulusTable=((K_f_3*(1-alternating_switch) + K_f_4*alternating_switch, ), ), definition=HYDRAULIC, fluidDensity=rho_3*(1-alternating_switch)+rho_4*alternating_switch, molecularWeight=None, useBulkModulus=True)
        Model_2D.interactionProperties['IntProp-' + cavity_name_4].setValues(bulkModulusTable=((K_f_4*(1-alternating_switch) + K_f_3*alternating_switch, ), ), definition=HYDRAULIC, fluidDensity=rho_4*(1-alternating_switch)+rho_3*alternating_switch, molecularWeight=None, useBulkModulus=True)
        Model_2D.FluidCavity(cavityPoint=Model_2D.rootAssembly.sets['RefPressure_' + append_name_1], cavitySurface=Model_2D.rootAssembly.surfaces['Surf-' + append_name_1], createStepName='Initial', interactionProperty='IntProp-' + cavity_name_1, name='Int-' + cavity_name_1)
        Model_2D.FluidCavity(cavityPoint=Model_2D.rootAssembly.sets['RefPressure_' + append_name_2], cavitySurface=Model_2D.rootAssembly.surfaces['Surf-' + append_name_2], createStepName='Initial', interactionProperty='IntProp-' + cavity_name_2, name='Int-' + cavity_name_2)
        Model_2D.FluidCavity(cavityPoint=Model_2D.rootAssembly.sets['RefPressure_' + append_name_3], cavitySurface=Model_2D.rootAssembly.surfaces['Surf-' + append_name_3], createStepName='Initial', interactionProperty='IntProp-' + cavity_name_3, name='Int-' + cavity_name_3)
        Model_2D.FluidCavity(cavityPoint=Model_2D.rootAssembly.sets['RefPressure_' + append_name_4], cavitySurface=Model_2D.rootAssembly.surfaces['Surf-' + append_name_4], createStepName='Initial', interactionProperty='IntProp-' + cavity_name_4, name='Int-' + cavity_name_4)




########################################################################
# -------------------- exchange properties -------------------
########################################################################
Model_2D.setValues(absoluteZero=absolute0, universalGas=universalGasConstant)
for cavity_hor_iter in range (1,no_holes_hor+1,2):
    for cavity_ver_iter in range (1,no_holes_ver+1,2):
        alternating_switch = 0.0
        if alternating_pattern == 1:
            if (cavity_ver_iter-1)/2%2==0:
                alternating_switch = 0.0
            else:
                alternating_switch = 1.0
        append_name_1 = 'r' + str(cavity_ver_iter) + 'c' + str(cavity_hor_iter)
        append_name_2 = 'r' + str(cavity_ver_iter) + 'c' + str(cavity_hor_iter+1)
        append_name_3 = 'r' + str(cavity_ver_iter+1) + 'c' + str(cavity_hor_iter)
        append_name_4 = 'r' + str(cavity_ver_iter+1) + 'c' + str(cavity_hor_iter+1)
        cavity_name_1 = 'FluidCavity_' + append_name_1
        cavity_name_2 = 'FluidCavity_' + append_name_2
        cavity_name_3 = 'FluidCavity_' + append_name_3
        cavity_name_4 = 'FluidCavity_' + append_name_4
        append_exchangename_13 = 'FluidExchange_' + append_name_1 + '-' + append_name_3
        append_exchangename_24 = 'FluidExchange_' + append_name_2 + '-' + append_name_4
        Model_2D.FluidExchangeProperty(dataTable=((0.0, 0.0), ((vol_flux_per_p_per_unitA_13*(1-alternating_switch)+vol_flux_per_p_per_unitA_24*alternating_switch)*1e10, 1.0*1e10)), definition=VOL_RATE_LEAK, name='IntProp-' + append_exchangename_13)
        Model_2D.FluidExchangeProperty(dataTable=((0.0, 0.0), ((vol_flux_per_p_per_unitA_24*(1-alternating_switch)+vol_flux_per_p_per_unitA_13*alternating_switch)*1e10, 1.0*1e10)), definition=VOL_RATE_LEAK, name='IntProp-' + append_exchangename_24)
# exchange definitions (seems that the first cavity must be the one that loses fluid with this definition of volume flux property!!!)
        Model_2D.FluidExchange(createStepName='Initial', definition=BETWEEN_CAVITIES, firstCavity='Int-' + cavity_name_3, secondCavity='Int-' + cavity_name_1, interactionProperty='IntProp-' + append_exchangename_13, name='Int-' + append_exchangename_13)
        Model_2D.FluidExchange(createStepName='Initial', definition=BETWEEN_CAVITIES, firstCavity='Int-' + cavity_name_2, secondCavity='Int-' + cavity_name_4, interactionProperty='IntProp-' + append_exchangename_24, name='Int-' + append_exchangename_24)


########################################################################
# ----------------------- Save Wet Model with flow -----------------------------
########################################################################

# Save as a Model
mdb.saveAs(pathName = JobNameModelwet  +'.cae')
print("saved wet model")




########################################################################
# ----------------------- Generate randomized node displacements -----------------------------
########################################################################
Output_file = open(ModelName + '_randomizednodedispl_new.py', 'w')
for i in range(len(Part_Full.nodes)):
    x_coord = Part_Full.nodes[i].coordinates[0]
    y_coord = Part_Full.nodes[i].coordinates[1]
    if abs(x_coord-x_bot) < TOL or abs(x_coord-x_top) < TOL  or abs(y_coord-y_bot) < TOL or abs(y_coord-y_top) < TOL : # check if boundary point
        random_imp_x = 0.0
        random_imp_y = 0.0
    else:
        random_imp_x = random.randint(-500, 500)/1000.0
        random_imp_y = random.randint(-500, 500)/1000.0
    Output_file.write('%10.8g \n' % (random_imp_x))
    Output_file.write('%10.8g \n' % (random_imp_y))
Output_file.close()
print("saved randomized node displacements")
