########################################################################
# --------------------------- Module Import ----------------------------
########################################################################
Mdb()
import os
from abaqus import *
from abaqusConstants import *
from assembly import *
from connectorBehavior import *
from interaction import *
from job import *
from load import *
from material import *
from mesh import *
from odbAccess import *
from part import *
from section import *
from sketch import *
from step import *
from visualization import *
from viewerModules import *
from numpy import *
from decimal import *
from operator import *
from string import *
import datetime
import math
import meshEdit
import os
import random
import shutil
import time
import visualization
import numpy
session.journalOptions.setValues(replayGeometry=COORDINATE,recoverGeometry=COORDINATE) # for python-oriented approach to the geometry
execfile('AbaqusScriptFunc.py')	# script to implement periodic BC easier



########################################################################
# ------------------------------ Variables -----------------------------
########################################################################

	# names
ModelName = 'ellipsesquirt' 
JobNameModel = ModelName + '-Model' 
JobNameModelwet = JobNameModel + '-wet'
JobNameSequential = ModelName + '-seq' 

Working_Dir = os.getcwd()
Scratch_Dir = Working_Dir

	# computing
num_cpu = 4
num_ram = 24
unit_ram = GIGA_BYTES

	# geometry [intended length unit: m]
scale_hole = 1.0
scale_aspect = 1.0
w_wall = 0.001e0 # wall thickness between cell boundary and hole boundary
d_cell = 0.030e0 # width of single cell
h_cell = 0.3*d_cell*scale_aspect # height of single cell
d_ellipse = d_cell-2.0*w_wall # horizontal ellipse diameter
h_ellipse = h_cell-2.0*w_wall # vertical ellipse diameter
depth = 1.0 # [m]
porosity = (pi*d_ellipse*h_ellipse)/(d_cell*h_cell)
no_holes_hor = 8 # number of holes in horizontal direction
no_holes_ver = no_holes_hor # number of holes in vertical direction

x_bot = -d_cell*1.0/2.0
y_bot = -h_cell*1.0/2.0
z_bot = 0.0
x_top = x_bot + d_cell*no_holes_hor
y_top = y_bot + h_cell*no_holes_ver
z_top = 0.0


# material
rho_mod = 5e-6

# solid
E0 = 1e5*1e2 # [Pa]
Nu0 = 0.45
Rho = 1000.0 # [kg/m^3]
solidname = 'solid'
Rho = Rho*rho_mod

# 	air
MW_air = 0.0289 # [kg/mol], molecular weight of air
absolute0 = 0.0 # reference point for Kelvin temperature scale
universalGasConstant = 8.31434 # universal gas constant
eta_air = 17.1e-6 # [Pa s], viscosity
K_air = 1.0e5 # [Pa], bulk modulus
rho_air = 1.2 # [kg/m^3]

# fluid
rho_fluid = 1000.0*1e-3 # [kg/m^3]
eta_fluid = eta_air
K_fluid = K_air
fluidname = 'fluid'

# for periodic patterns of
# --------------------
# |(3)||(4)||(3)||(4)|
# --------------------
# |(1)||(2)||(1)||(2)|
# --------------------
# --------------------
# |(3)||(4)||(3)||(4)|
# --------------------
# |(1)||(2)||(1)||(2)|
# --------------------
# alternating pattern (0.0 = normal, 1.0 = every other 4-group will change fluid properties as 1<->2 and 3<->4)
alternating_pattern = 0
# cavities are equal in all simulations, the following options yet allow to assign individual properties
# lower left cavity (1)
K_f_1 = K_fluid;
rho_1 = rho_fluid;
# lower right cavity (2)
K_f_2 = K_fluid;
rho_2 = rho_fluid;
# upper left cavity (3)
K_f_3 = K_fluid;
rho_3 = rho_fluid;
# upper right cavity (4)
K_f_4 = K_fluid;
rho_4 = rho_fluid;
# connection left (1-3)
eta_13 = eta_fluid;
A_exchange_13 = 2.895e-7 # [m^2] - cross sectional area for a tube radius of 0.3e-3 [m]
perm_13 = 1.152e-8 # [m^2] permeability for a tube radius of 0.3e-3 [m]
d_connect_13 = w_wall*2.0 # [m], connection length
vol_flux_per_p_per_unitA_13 = perm_13/(eta_13*d_connect_13)*A_exchange_13 # volume flux / (pressure difference)
	# cf.: Error in job ellipsesquirt-PB: THE EFFECTIVE AREA PARAMETER IS NOT AVAILABLE FOR Abaqus/Standard <-- It seems the "effective area" has to be unset and remains at 1.0
# connection right (2-4)
eta_24 = eta_13;
A_exchange_24 = A_exchange_13
perm_24 = perm_13
d_connect_24 = d_connect_13
vol_flux_per_p_per_unitA_24 = perm_24/(eta_24*d_connect_24)*A_exchange_24 # volume flux / (pressure difference)
	# cf.: Error in job ellipsesquirt-PB: THE EFFECTIVE AREA PARAMETER IS NOT AVAILABLE FOR Abaqus/Standard <-- It seems the "effective area" has to be unset and remains at 1.0

V_1_0 = pi/4.0*d_ellipse*h_ellipse*depth
V_2_0 = V_1_0
f_c = 1.0/(2.0*pi)*min(K_f_1,K_f_2,K_f_3,K_f_4)*min((perm_13*A_exchange_13)/(eta_13*d_connect_13),(perm_24*A_exchange_24)/(eta_24*d_connect_24))*(1.0/V_1_0+1.0/V_2_0)


########################################################################
# ------------------------------- Meshing ------------------------------
########################################################################
M_Size = 0.35*w_wall
Min_M_Size = 0.3
M_Devi = 0.3
TOL = 1e-5


########################################################################
# ---------------------------- Solver Settings -------------------------
########################################################################

# sequence control
no_seq = 11
# quasistatic deformations
t_deform = 30.0/f_c
t_relax_def = 20.0/f_c
incrementnumber_max = 120000 
incrementstep_min = 1e-12
incrementstep_initial = 1e-6*t_deform
incrementstep_max = t_deform
no_output_interv_def = 40
strain_total = -0.11 # final postbuckling engineering strain for quasistatic deformation
bc_disp_total = strain_total * (y_top-y_bot)
IMP_RD = 0.0*w_wall # scaling for random pre-imperfections
# dynamic linear perturbations
perturb_amp = 1e-3
t_relax = 3.5/f_c # time of post-buckling simulation for dynamic linear and non-linear deformation
t_load = t_relax*1e-2 # step time of loading/deformation
no_output_interv_dyn = 200
# misc
linbulkvisc_expl = 0.06
quadbulkvisc_expl = 1.2
